﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace dcf77_send {
    class dcf77_encode {
        //http://bastianborn.de/funkuhr-hack
        //http://www.ulrichradig.de/home/index.php/avr/dcf77_uhr
        //http://rn-wissen.de/wiki/index.php/DCF77-Decoder_als_Bascom-Library
        //http://www.rrs-web.net/in3her/dcf77_32.html

        public static byte[] calcDcf77Bytes(DateTime dt) {
            byte[] data = new byte[60];

            byte minute = (byte)dt.Minute;
            byte hour = (byte)dt.Hour;

            byte day = (byte)dt.Day;
            byte weekday = (byte)dt.DayOfWeek;
            byte month = (byte)dt.Month;
            byte year = (byte)(dt.Year - (Convert.ToInt32(dt.Year.ToString().Substring(0, 2)) * 100));

            byte minutebcd = (byte)helper.byteToBcd(minute);
            byte hourbcd = (byte)helper.byteToBcd(hour);

            byte daybcd = (byte)helper.byteToBcd(day);
            byte weekdaybcd = (byte)helper.byteToBcd(weekday);
            byte monthbcd = (byte)helper.byteToBcd(month);
            byte yearbcd = (byte)helper.byteToBcd(year);

            bool parityminute = false;
            bool parityhour = false;
            bool paritydate = false;

            byte summertime = (byte)(dt.IsDaylightSavingTime() ? 1 : 0);
            byte summertimechange = (byte)((dt.Add(new TimeSpan(1, 0, 0)).IsDaylightSavingTime() != dt.IsDaylightSavingTime()) ? 1 : 0);

            byte switchsek = 0;

            for (int i = 0; i < 60; i++) {
                if (i == 0) {
                    //0
                    data[i] = 0;
                }
                if (i >= 1 && i <= 14) {
                    //Wetter (verschlüsselt)
                    data[i] = 0;
                }
                if (i == 15) {
                    //Antenne
                    data[i] = 0;
                }
                if (i == 16) {
                    //Ende der Stunde Wird MEZ/MESZ umgestellt
                    data[i] = summertimechange;
                }
                if (i >= 17 && i <= 18) {
                    //Sommer/Winterzeit   MESZ=10 MEZ=01
                    if (summertime == 1) {
                        data[i] = 1;
                        data[i + 1] = 0;
                    } else {
                        data[i] = 0;
                        data[i + 1] = 1;
                    }

                    i = 19;
                }
                if (i == 19) {
                    //Am Ende der Stunde wird Schaltsek angefügt
                    data[i] = switchsek;
                }
                if (i == 20) {
                    //Startbit

                    data[i] = 1;
                }
                if (i >= 21 && i <= 27) {
                    //Minute
                    byte tmp;
                    tmp = (byte)(minutebcd & 1);
                    data[i] = tmp;
                    if (tmp == 1) parityminute = !parityminute;
                    tmp = (byte)((minutebcd & 2) >> 1);
                    data[i + 1] = tmp;
                    if (tmp == 1) parityminute = !parityminute;
                    tmp = (byte)((minutebcd & 4) >> 2);
                    data[i + 2] = tmp;
                    if (tmp == 1) parityminute = !parityminute;
                    tmp = (byte)((minutebcd & 8) >> 3);
                    data[i + 3] = tmp;
                    if (tmp == 1) parityminute = !parityminute;
                    tmp = (byte)((minutebcd & 16) >> 4);
                    data[i + 4] = tmp;
                    if (tmp == 1) parityminute = !parityminute;
                    tmp = (byte)((minutebcd & 32) >> 5);
                    data[i + 5] = tmp;
                    if (tmp == 1) parityminute = !parityminute;
                    tmp = (byte)((minutebcd & 64) >> 6);
                    data[i + 6] = tmp;
                    if (tmp == 1) parityminute = !parityminute;

                    i = 28;
                }
                if (i == 28) {
                    //Parität Minute
                    data[i] = (byte)(parityminute ? 1 : 0);
                }
                if (i >= 29 && i <= 34) {
                    //Stunde
                    byte tmp;
                    tmp = (byte)(hourbcd & 1);
                    data[i] = tmp;
                    if (tmp == 1) parityhour = !parityhour;
                    tmp = (byte)((hourbcd & 2) >> 1);
                    data[i + 1] = tmp;
                    if (tmp == 1) parityhour = !parityhour;
                    tmp = (byte)((hourbcd & 4) >> 2);
                    data[i + 2] = tmp;
                    if (tmp == 1) parityhour = !parityhour;
                    tmp = (byte)((hourbcd & 8) >> 3);
                    data[i + 3] = tmp;
                    if (tmp == 1) parityhour = !parityhour;
                    tmp = (byte)((hourbcd & 16) >> 4);
                    data[i + 4] = tmp;
                    if (tmp == 1) parityhour = !parityhour;
                    tmp = (byte)((hourbcd & 32) >> 5);
                    data[i + 5] = tmp;
                    if (tmp == 1) parityhour = !parityhour;

                    i = 35;
                }
                if (i == 35) {
                    //Parität Stunde
                    data[i] = (byte)(parityhour ? 1 : 0);
                }
                if (i >= 36 && i <= 41) {
                    //Tag
                    byte tmp;
                    tmp = (byte)(daybcd & 1);
                    data[i] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((daybcd & 2) >> 1);
                    data[i + 1] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((daybcd & 4) >> 2);
                    data[i + 2] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((daybcd & 8) >> 3);
                    data[i + 3] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((daybcd & 16) >> 4);
                    data[i + 4] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((daybcd & 32) >> 5);
                    data[i + 5] = tmp;
                    if (tmp == 1) paritydate = !paritydate;

                    i = 42;
                }
                if (i >= 42 && i <= 44) {
                    //Tag der Woche
                    byte tmp;
                    tmp = (byte)(weekdaybcd & 1);
                    data[i] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((weekdaybcd & 2) >> 1);
                    data[i + 1] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((weekdaybcd & 4) >> 2);
                    data[i + 2] = tmp;
                    if (tmp == 1) paritydate = !paritydate;

                    i = 45;
                }
                if (i >= 45 && i <= 49) {
                    //Monat
                    byte tmp;
                    tmp = (byte)(monthbcd & 1);
                    data[i] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((monthbcd & 2) >> 1);
                    data[i + 1] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((monthbcd & 4) >> 2);
                    data[i + 2] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((monthbcd & 8) >> 3);
                    data[i + 3] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((monthbcd & 16) >> 4);
                    data[i + 4] = tmp;
                    if (tmp == 1) paritydate = !paritydate;

                    i = 50;
                }
                if (i >= 50 && i <= 57) {
                    //Jahr
                    byte tmp;
                    tmp = (byte)(yearbcd & 1);
                    data[i] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((yearbcd & 2) >> 1);
                    data[i + 1] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((yearbcd & 4) >> 2);
                    data[i + 2] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((yearbcd & 8) >> 3);
                    data[i + 3] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((yearbcd & 16) >> 4);
                    data[i + 4] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((yearbcd & 32) >> 5);
                    data[i + 5] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((yearbcd & 64) >> 6);
                    data[i + 6] = tmp;
                    if (tmp == 1) paritydate = !paritydate;
                    tmp = (byte)((yearbcd & 128) >> 7);
                    data[i + 7] = tmp;
                    if (tmp == 1) paritydate = !paritydate;

                    i = 58;
                }
                if (i == 58) {
                    //Parität des Datums
                    data[i] = (byte)(paritydate ? 1 : 0);
                }
                if (i == 59) {
                    //Sync
                    data[i] = 255;
                }
            }
            return data;
        }
    }
}
